﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using GisdkAPI;

namespace BatchMaps
{
    public class Builder
    {
        // Points layer name to use for choosing locations.
        private const string pointsLayer = "Stores";
        // Field name of name field in point_layer to be used for displaying a list.
        private const string nameField = "Name";

        // Width of map in miles; map scope will be set to this width.
        private const double map_width_miles = 1.0;

        // Position of map on layout in inches from upper left hand corner.
        // Note: using inches here makes it easier to specify locations.  Later
        // they will be converted to layout dimensions which are in hundredths of
        // points (7,200 per inch).
        private const double map_ulx = 0.5;       // upper left x position
        private const double map_uly = 1.0;       // upper left y position
        private const double map_lrx = 8.5;       // lower right x position
        private const double map_lry = 7.5;       // lower right y position

        // Upper left hand corner of legend on layout in inches from upper left
        // hand corner of layout.
        private const double legend_ulx = 8.75;   // upper left x position
        private const double legend_uly = 1.0;    // upper left y position

        // Position of title layout in inches from upper left hand corner.
        private const double title_ulx = 3.0;     // upper left x position
        private const double title_uly = .5;      // upper left y position
        private const double title_lrx = 5.0;     // lower right x position
        private const double title_lry = .8;      // lower right y position

        private dynamic gisdk;

        public Builder(IGisdkEngine app)
        {
            gisdk = app;
        }

        public IList<Feature> ListFeatures()
        {
            var features = new List<Feature>();
            var order = new OptionsArray();
            order[nameField] = "Ascending";
            var rec = gisdk.GetFirstRecord(pointsLayer + "|", order);
            while (rec != null)
            {
                var values = gisdk.GetRecordValues(pointsLayer, rec, new[] {nameField});
                var feature = new Feature() {Name = values[0][1], RecordHandle = rec};
                features.Add(feature);

                rec = gisdk.GetNextRecord(pointsLayer + "|", rec, order);
            }
            return features;
        }

        public void DrawMaps(IList<Feature> selectedFeatures, bool testMode)
        {
            // Set the map aspect ratio (width divided by height) to fit that of the
            // location rectangle specified for the map on the layout.

            // Get the current map window size.
            var sz = gisdk.GetWindowSize(null);
            var map_window_wd = sz[0];  // width as percent
            var map_window_ht = sz[1];  // height as percent

            // Compute aspect ratio of map on layout.
            var layout_ar = (map_lrx - map_ulx)/(map_lry - map_uly);

            // Set the window to 600 pixels wide, and adjust the height to match the
            // aspect ratio on the layout.
            gisdk.SetWindowSizePixels(null, 600, (int) (600/layout_ar));

            // Create location variables for use in AddAnnotation().
            // All locations on a layout are specified in hundredths of points, so you
            // must convert the inches in the Administrative Settings section to
            // hundredths of points and create location arrays containing two coordinates
            // which form a rectangle.
            gisdk.SetMapUnits("Miles");
            var ppi = 7200;  // layout units (hundredths of points) per inch
            var map_loc = new object[]
                {
                    gisdk.Coord((int) (map_ulx*ppi), (int) (map_uly*ppi)),
                    gisdk.Coord((int) (map_lrx*ppi), (int) (map_lry*ppi))
                };
            // Only need upper left corner of legend
            var legend_loc = gisdk.Coord((int) (legend_ulx*ppi), (int) (legend_uly*ppi));
            var title_loc = new object[]
                {
                    gisdk.Coord((int) (title_ulx*ppi), (int) (title_uly*ppi)),
                    gisdk.Coord((int) (title_lrx*ppi), (int) (title_lry*ppi))
                };

            // Set up the map boder style variables.
            var solidline = gisdk.LineStyle(new[] {new[] {new[] {1, -1, 0}}});
            var black = gisdk.ColorRGB(0, 0, 0);

            // Get the map name.
            string mapname = gisdk.GetMap();

            // Create an empty layout to use for the batch printing.
            string layout_name = "batchlayout";
            gisdk.CreateLayout(layout_name, null);

            // Loop through the selectedFeatures array, create a layout, and print it if not
            // in test mode.
            gisdk.SetLayer(pointsLayer);
            if (!testMode)
                gisdk.CreateProgressBar(null, "True");  // True provides a Cancel button
            var featureIndex = 0;

            foreach (Feature feature in selectedFeatures)
            {
                featureIndex++;
                if (!testMode)
                {
                    if (gisdk.UpdateProgressBar(String.Format("Printing layout {0} of {1}...",
                                                                            featureIndex, selectedFeatures.Count), (int)(100.0 * (featureIndex - 1) / selectedFeatures.Count)) == "True")
                        break;
                }

                // Get the point's record handle to center the map on the point.
                var c = gisdk.GetPoint(gisdk.RecordHandleToId(feature.RecordHandle));

                // Set the map scope based on the point and the width set in
                // the Administrative Settings, then redraw the map.
                gisdk.SetMapScope(mapname, gisdk.Scope(c, map_width_miles, map_width_miles/layout_ar, 0));
                gisdk.RedrawMap(mapname);

                // add map to the layout.
                var mapOptions = new OptionsArray();
                mapOptions["Location"] = map_loc;
                mapOptions["Map"] = mapname;
                mapOptions["Scope"] = gisdk.GetMapScope(mapname);
                mapOptions["Always Redraw"] = "True";
                mapOptions["Drop Shadow"] = "False";
                mapOptions["LineWidth"] = .5;
                mapOptions["Line Color"] = black;
                mapOptions["LineStyle"] = solidline;
                var mapid = gisdk.AddAnnotation(null, "Map", mapOptions);

                // Get the name of the map on the layout to use in adding the legend.
                object[] newmapdef = gisdk.GetAnnotation(null, mapid);
                var legend_map_name = new OptionsArray(newmapdef[1])["Map"];

                // Add the legend to the layout
                var legendOptions = new OptionsArray();
                // This is the map name on the layout, not the original map name.
                legendOptions["Name"] = legend_map_name;
                // only need to specify the upper left corner
                legendOptions["Location"] = legend_loc;
                gisdk.AddAnnotation(null, "Legend", legendOptions);

                // Set the record so you can get the current name to use for title.
                gisdk.SetRecord(pointsLayer, feature.RecordHandle);

                // Add a title using the name field.
                var titleOptions = new OptionsArray();
                titleOptions["Color"] = black;
                titleOptions["Angle"] = 0;
                titleOptions["Font"] = "Arial|Bold|18";
                titleOptions["Location"] = title_loc;
                titleOptions["Text"] = feature.Name;  // use name field for title
                var titleid = gisdk.AddAnnotation(null, "Text", titleOptions);

                // Redraw the layout.
                gisdk.DrawLayout(null);

                // Print the layout, or display a message if in test mode.
                if (!testMode)
                    gisdk.PrintLayout(1, 1);
                else if (MessageBox.Show(gisdk.GetMainWindow(), String.Format("Layout {0} of {1}", featureIndex, selectedFeatures.Count), 
                        "Continue?", MessageBoxButtons.YesNo) == DialogResult.No)
                    break;

                // Drop everything on the layout.
                gisdk.DropAnnotations(null, gisdk.GetAnnotations(null));
            }

            if (!testMode)
                gisdk.DestroyProgressBar();

            // Close the layout. You could remove this statement to leave the layout open.
            gisdk.CloseLayout(layout_name);

            // Display completion message to user.
            if (!testMode)
                MessageBox.Show(String.Format("Printed {0} layouts.  Completed at {1}.", selectedFeatures.Count, DateTime.Now));

        }
    }
}